/*
 * This file contains the user specified measurement functions for the different integration regions
 * for the uncorrelated emission colour structure.
 * The recipe to derive the measurement functions F from the definition of the observable can be found 
 * in the manual.
 */

#include <Auxiliaries.h>
#include <Input_Common.h>
#include <Input_Parameters.h>
#include <cmath>
#include <iostream>

#ifdef BOOSTEDSOURCE_H_
using namespace boost::multiprecision;
using namespace boost::math;
#endif /* BOOSTEDSOURCE_H_ */

/*   __  __                                                    _      __                  _   _
 *  |  \/  |                                                  | |    / _|                | | (_)
 *  | \  / | ___  __ _ ___ _   _ _ __ ___ _ __ ___   ___ _ __ | |_  | |_ _   _ _ __   ___| |_ _  ___  _ __  ___
 *  | |\/| |/ _ \/ _` / __| | | | '__/ _ \ '_ ` _ \ / _ \ '_ \| __| |  _| | | | '_ \ / __| __| |/ _ \| '_ \/ __|
 *  | |  | |  __/ (_| \__ \ |_| | | |  __/ | | | | |  __/ | | | |_  | | | |_| | | | | (__| |_| | (_) | | | \__ \
 *  |_|  |_|\___|\__,_|___/\__,_|_|  \___|_| |_| |_|\___|_| |_|\__| |_|  \__,_|_| |_|\___|\__|_|\___/|_| |_|___/
 */

/*  _    _                               _       _           _ 
 * | |  | |                             | |     | |         | |
 * | |  | |_ __   ___ ___  _ __ _ __ ___| | __ _| |_ ___  __| |
 * | |  | | '_ \ / __/ _ \| '__| '__/ _ \ |/ _` | __/ _ \/ _` |
 * | |__| | | | | (_| (_) | |  | | |  __/ | (_| | ||  __/ (_| |
 *  \____/|_| |_|\___\___/|_|  |_|  \___|_|\__,_|\__\___|\__,_|
 */


double GAa(decimal r,decimal y,decimal tkl,decimal tk,decimal tl,decimal b) {
	// Please use the predefined variables tkl (=t_kl), ck (=cos theta_k) or tk (=(1-ck)/2), cl (=cos theta_l) or tl (=(1-cl)/2) for the definitions below
	decimal ck=Ck(tk);
	decimal cl=Cl(tl);
	decimal GAa=0;

	/* Please input the C++ syntax measurement function G_Aa here. As a reminder: G_Aa you get by using the
	 * parametrisation for the uncorrelated emission from the manual to the exponent of the full 
	 * measurement exponential, resubstituting yk-> r*y, yl->y and discarding a few factors.
	 *
	 * Don't forget to capture all zeroes and integrable divergences via if-clauses that set GAa to a constant.
 	 */ 
	GAa=2.*abs(b*ck+cl)*pow(1.+b,-1);
	if (b*ck+cl==0.) {
		GAa=1.;
	}


	// Automated error message if a zero/divergence is not defused, please leave this as it is
	if (GAa<=0. || isnan(GAa) || isinf(GAa)) {
		cerr<<"Measurement function zero or NaN in region A, subregion a with value "<<GAa<<" at y="<<y<<" r="<<r<<" b="<<b<<" tkl="<<tkl<<" tk="<<tk<<" tl="<<tl<<endl;
		GAa=1.;
	}

	return ReturnDouble(GAa);
}

double GAb(decimal r,decimal y,decimal tkl,decimal tk,decimal tl,decimal b) {
	// Please use the predefined variables tkl (=t_kl), ck (=cos theta_k) or tk (=(1-ck)/2), cl (=cos theta_l) or tl (=(1-cl)/2) for the definitions below
	decimal ck=Ck(tk);
	decimal cl=Cl(tl);
	decimal GAb=0;

	/* Please input the C++ syntax measurement function G_Ab here. As a reminder: G_Ab you get by using the
	 * parametrisation for the uncorrelated emission from the manual to the exponent of the full 
	 * measurement exponential, resubstituting yl-> r*y, yk->y (yk, yl swapped compared to GAa) and discarding a few factors.
	 *
	 * Don't forget to capture all zeroes and integrable divergences via if-clauses that set GAb to a constant.
 	 */ 
	GAb=GAa(r,y,tkl,tk,tl,b);

	

	// Automated error message if a zero/divergence is not defused, please leave this as it is
	if (GAb<=0. || isnan(GAb) || isinf(GAb)) {
		cerr<<"Measurement function zero or NaN in region A, subregion b with value "<<GAb<<" at y="<<y<<" r="<<r<<" b="<<b<<" tkl="<<tkl<<" tk="<<tk<<" tl="<<tl<<endl;
		GAb=1.;
	}

	return ReturnDouble(GAb);
}

double GB(decimal yk,decimal yl,decimal tkl,decimal tk,decimal tl,decimal b) {
	// Please use the predefined variables tkl (=t_kl), ck (=cos theta_k) or tk (=(1-ck)/2), cl (=cos theta_l) or tl (=(1-cl)/2) for the definitions below
	decimal ck=Ck(tk);
	decimal cl=Cl(tl);
	decimal GB=0;

	/* Please input the C++ syntax measurement function G_B here. As a reminder: G_B you get by using the
	 * parametrisation for the uncorrelated emission from the manual to the exponent of the full 
	 * measurement exponential, inverting ONE of EITHER yk or yl, and discarding a few factors. 
	 *
	 * Don't forget to capture all zeroes and integrable divergences via if-clauses that set GB to a constant.
 	 */ 
	GB=2.*abs(b*ck+cl)*pow(1.+b,-1);
	if (b*ck+cl==0.) {
		GB=1.;
	}

	// Automated error message if a zero/divergence is not defused, please leave this as it is
	if (GB<=0. || isnan(GB) || isinf(GB)) {
		cerr<<"Measurement function zero or NaN in region B with value "<<GB<<" at yk="<<yk<<" yl="<<yl<<" b="<<b<<" tkl="<<tkl<<" tk="<<tk<<" tl="<<tl<<endl;
		GB=1.;
	}

	return ReturnDouble(GB);
}

/*This file is part of SoftSERVE.
 *
 *   SoftSERVE is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   SoftSERVE is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with SoftSERVE.  If not, see <http://www.gnu.org/licenses/>.
 */
