/*This file is part of SoftSERVE.
 *
 *   SoftSERVE is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   SoftSERVE is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with SoftSERVE.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * This file contains the difference structures arising from plus distributions tying together the Measurement functions FXi, plus distribution denominators 
 * and contributing remainder/numerator functions XCfEi for the 5D integration at all eps-orders of the Cf^2 structure, subregion a. 
 * Example: I5EM1s ('I'ntegration over '5' dimensions, at 'E'ps to the 'M'inus '1'st order, only missing integration over 's'5) contains various logs of FXi functions
 * evaluated at s5=0, i.e. with only the delta from the s5 +-distribution, multiplied with the correct X functions' eps-powers to arrive at eps^-1. 
 */

#include <5Int.h>
#include <Auxiliaries.h>
#include <cmath>
#include <XFactor.h>
#include <Input_Common.h>

using namespace std;

double I5EM1r(double c, double x, double s, double s5, double s6) {
	return 0; //alpha pole present, leading order x-indep
}
double I5E0r(double c, double x, double s, double s5, double s6) {
	return -4*pow(c,-1)*pow(mexp,-1)*pow(n,-1)*pow(s5,-1)*pow(x,-1)*((log(FA1a(0,0,t(s),0,tl(s6),0)) - log(FA1a(0,0,t(s),0,tl(s6),b(c))) - log(FA1a(0,y(x),t(s),0,tl(s6),0)) + log(FA1a(0,y(x),t(s),0,tl(s6),b(c))) +
	        log(FA1b(0,0,t(s),0,tl(s6),0)) - log(FA1b(0,0,t(s),0,tl(s6),b(c))) - log(FA1b(0,y(x),t(s),0,tl(s6),0)) + log(FA1b(0,y(x),t(s),0,tl(s6),b(c))) + log(FA2a(0,0,t(s),0,tl(s6),0)) -
	        log(FA2a(0,0,t(s),0,tl(s6),b(c))) - log(FA2a(0,y(x),t(s),0,tl(s6),0)) + log(FA2a(0,y(x),t(s),0,tl(s6),b(c))) + log(FA2b(0,0,t(s),0,tl(s6),0)) - log(FA2b(0,0,t(s),0,tl(s6),b(c))) -
	        log(FA2b(0,y(x),t(s),0,tl(s6),0)) + log(FA2b(0,y(x),t(s),0,tl(s6),b(c))))*XCfE0(s,0,s6) +
	     (-log(FA1a(0,0,t(s),t5(s5),tl(s6),0)) + log(FA1a(0,0,t(s),t5(s5),tl(s6),b(c))) + log(FA1a(0,y(x),t(s),t5(s5),tl(s6),0)) - log(FA1a(0,y(x),t(s),t5(s5),tl(s6),b(c))) - log(FA1b(0,0,t(s),t5(s5),tl(s6),0)) +
	        log(FA1b(0,0,t(s),t5(s5),tl(s6),b(c))) + log(FA1b(0,y(x),t(s),t5(s5),tl(s6),0)) - log(FA1b(0,y(x),t(s),t5(s5),tl(s6),b(c))) - log(FA2a(0,0,t(s),t5(s5),tl(s6),0)) + log(FA2a(0,0,t(s),t5(s5),tl(s6),b(c))) +
	        log(FA2a(0,y(x),t(s),t5(s5),tl(s6),0)) - log(FA2a(0,y(x),t(s),t5(s5),tl(s6),b(c))) - log(FA2b(0,0,t(s),t5(s5),tl(s6),0)) + log(FA2b(0,0,t(s),t5(s5),tl(s6),b(c))) + log(FA2b(0,y(x),t(s),t5(s5),tl(s6),0)) -
	        log(FA2b(0,y(x),t(s),t5(s5),tl(s6),b(c))))*XCfE0(s,s5,s6));
}

double I5EM1x(double c, double r, double s, double s5, double s6) {
	return 0; //alpha pole present, leading order r-indep
}
double I5E0x(double c, double r, double s, double s5, double s6) {
	return -2*pow(c,-1)*pow(mexp,-1)*pow(n,-1)*pow(r,-1)*pow(s5,-1)*((log(FA1a(0,0,t(s),0,tl(s6),0)) - log(FA1a(0,0,t(s),0,tl(s6),b(c))) - log(FA1a(q(r),0,t(s),0,tl(s6),0)) + log(FA1a(q(r),0,t(s),0,tl(s6),b(c))) +
	        log(FA1b(0,0,t(s),0,tl(s6),0)) - log(FA1b(0,0,t(s),0,tl(s6),b(c))) - log(FA1b(q(r),0,t(s),0,tl(s6),0)) + log(FA1b(q(r),0,t(s),0,tl(s6),b(c))) + log(FA2a(0,0,t(s),0,tl(s6),0)) -
	        log(FA2a(0,0,t(s),0,tl(s6),b(c))) - log(FA2a(q(r),0,t(s),0,tl(s6),0)) + log(FA2a(q(r),0,t(s),0,tl(s6),b(c))) + log(FA2b(0,0,t(s),0,tl(s6),0)) - log(FA2b(0,0,t(s),0,tl(s6),b(c))) -
	        log(FA2b(q(r),0,t(s),0,tl(s6),0)) + log(FA2b(q(r),0,t(s),0,tl(s6),b(c))))*XCfE0(s,0,s6) +
	     (-log(FA1a(0,0,t(s),t5(s5),tl(s6),0)) + log(FA1a(0,0,t(s),t5(s5),tl(s6),b(c))) + log(FA1a(q(r),0,t(s),t5(s5),tl(s6),0)) - log(FA1a(q(r),0,t(s),t5(s5),tl(s6),b(c))) - log(FA1b(0,0,t(s),t5(s5),tl(s6),0)) +
	        log(FA1b(0,0,t(s),t5(s5),tl(s6),b(c))) + log(FA1b(q(r),0,t(s),t5(s5),tl(s6),0)) - log(FA1b(q(r),0,t(s),t5(s5),tl(s6),b(c))) - log(FA2a(0,0,t(s),t5(s5),tl(s6),0)) + log(FA2a(0,0,t(s),t5(s5),tl(s6),b(c))) +
	        log(FA2a(q(r),0,t(s),t5(s5),tl(s6),0)) - log(FA2a(q(r),0,t(s),t5(s5),tl(s6),b(c))) - log(FA2b(0,0,t(s),t5(s5),tl(s6),0)) + log(FA2b(0,0,t(s),t5(s5),tl(s6),b(c))) + log(FA2b(q(r),0,t(s),t5(s5),tl(s6),0)) -
	        log(FA2b(q(r),0,t(s),t5(s5),tl(s6),b(c))))*XCfE0(s,s5,s6));
}

double I5EM1c(double r, double x, double s, double s5, double s6) {
	return 0; //leading order is xi-indep
}
double I5E0c(double r, double x, double s, double s5, double s6) {
	return pow(r,-1)*pow(s5,-1)*pow(x,-1)*((log(FA1a(0,0,t(s),0,tl(s6),0)) - log(FA1a(0,y(x),t(s),0,tl(s6),0)) - log(FA1a(q(r),0,t(s),0,tl(s6),0)) + log(FA1a(q(r),y(x),t(s),0,tl(s6),0)) + log(FA1b(0,0,t(s),0,tl(s6),0)) -
	        log(FA1b(0,y(x),t(s),0,tl(s6),0)) - log(FA1b(q(r),0,t(s),0,tl(s6),0)) + log(FA1b(q(r),y(x),t(s),0,tl(s6),0)) + log(FA2a(0,0,t(s),0,tl(s6),0)) - log(FA2a(0,y(x),t(s),0,tl(s6),0)) -
	        log(FA2a(q(r),0,t(s),0,tl(s6),0)) + log(FA2a(q(r),y(x),t(s),0,tl(s6),0)) + log(FA2b(0,0,t(s),0,tl(s6),0)) - log(FA2b(0,y(x),t(s),0,tl(s6),0)) - log(FA2b(q(r),0,t(s),0,tl(s6),0)) +
	        log(FA2b(q(r),y(x),t(s),0,tl(s6),0)))*XCfE0(s,0,s6) + (-log(FA1a(0,0,t(s),t5(s5),tl(s6),0)) + log(FA1a(0,y(x),t(s),t5(s5),tl(s6),0)) + log(FA1a(q(r),0,t(s),t5(s5),tl(s6),0)) -
	        log(FA1a(q(r),y(x),t(s),t5(s5),tl(s6),0)) - log(FA1b(0,0,t(s),t5(s5),tl(s6),0)) + log(FA1b(0,y(x),t(s),t5(s5),tl(s6),0)) + log(FA1b(q(r),0,t(s),t5(s5),tl(s6),0)) -
	        log(FA1b(q(r),y(x),t(s),t5(s5),tl(s6),0)) - log(FA2a(0,0,t(s),t5(s5),tl(s6),0)) + log(FA2a(0,y(x),t(s),t5(s5),tl(s6),0)) + log(FA2a(q(r),0,t(s),t5(s5),tl(s6),0)) -
	        log(FA2a(q(r),y(x),t(s),t5(s5),tl(s6),0)) - log(FA2b(0,0,t(s),t5(s5),tl(s6),0)) + log(FA2b(0,y(x),t(s),t5(s5),tl(s6),0)) + log(FA2b(q(r),0,t(s),t5(s5),tl(s6),0)) - log(FA2b(q(r),y(x),t(s),t5(s5),tl(s6),0)))
	       *XCfE0(s,s5,s6) + XCfE1(0,0,s,0,s6) - XCfE1(0,0,s,s5,s6) - XCfE1(0,x,s,0,s6) + XCfE1(0,x,s,s5,s6) - XCfE1(r,0,s,0,s6) + XCfE1(r,0,s,s5,s6) + XCfE1(r,x,s,0,s6) - XCfE1(r,x,s,s5,s6));
}

double I5EM1s(double c, double r, double x, double s, double s6) {
	return 0; //eps pole present, all other orders at leading -> vanishes
}
double I5E0s(double c, double r, double x, double s, double s6) {
	return 2*(log(FA1a(0,0,t(s),0,tl(s6),0)) - log(FA1a(0,0,t(s),0,tl(s6),b(c))) - log(FA1a(0,y(x),t(s),0,tl(s6),0)) + log(FA1a(0,y(x),t(s),0,tl(s6),b(c))) - log(FA1a(q(r),0,t(s),0,tl(s6),0)) +
		     log(FA1a(q(r),0,t(s),0,tl(s6),b(c))) + log(FA1a(q(r),y(x),t(s),0,tl(s6),0)) - log(FA1a(q(r),y(x),t(s),0,tl(s6),b(c))) + log(FA1b(0,0,t(s),0,tl(s6),0)) - log(FA1b(0,0,t(s),0,tl(s6),b(c))) -
		     log(FA1b(0,y(x),t(s),0,tl(s6),0)) + log(FA1b(0,y(x),t(s),0,tl(s6),b(c))) - log(FA1b(q(r),0,t(s),0,tl(s6),0)) + log(FA1b(q(r),0,t(s),0,tl(s6),b(c))) + log(FA1b(q(r),y(x),t(s),0,tl(s6),0)) -
		     log(FA1b(q(r),y(x),t(s),0,tl(s6),b(c))) + log(FA2a(0,0,t(s),0,tl(s6),0)) - log(FA2a(0,0,t(s),0,tl(s6),b(c))) - log(FA2a(0,y(x),t(s),0,tl(s6),0)) + log(FA2a(0,y(x),t(s),0,tl(s6),b(c))) -
		     log(FA2a(q(r),0,t(s),0,tl(s6),0)) + log(FA2a(q(r),0,t(s),0,tl(s6),b(c))) + log(FA2a(q(r),y(x),t(s),0,tl(s6),0)) - log(FA2a(q(r),y(x),t(s),0,tl(s6),b(c))) + log(FA2b(0,0,t(s),0,tl(s6),0)) -
		     log(FA2b(0,0,t(s),0,tl(s6),b(c))) - log(FA2b(0,y(x),t(s),0,tl(s6),0)) + log(FA2b(0,y(x),t(s),0,tl(s6),b(c))) - log(FA2b(q(r),0,t(s),0,tl(s6),0)) + log(FA2b(q(r),0,t(s),0,tl(s6),b(c))) +
		     log(FA2b(q(r),y(x),t(s),0,tl(s6),0)) - log(FA2b(q(r),y(x),t(s),0,tl(s6),b(c))))*pow(c,-1)*pow(r,-1)*pow(x,-1)*XCfE0(s,0,s6);
}
