/*
 * This file contains the difference structures arising from plus distributions tying together the Measurement functions FXi, plus distribution denominators 
 * and contributing remainder/numerator functions XPNfij for the 4D integration at all eps-orders and the alpha^0 order of the pseudo-nf structure. 
 * Example: PNfI4EM1s ('P'seudo-'Nf' structure, 'I'ntegration over '4' dimensions, at 'E'ps to the 'M'inus '1'st and alpha^0 order, fourth integration over 's'5) 
 * contains logs of FXi functions evaluated at z=0,x=0, i.e. with the deltas from all +-distributions except the t5/s5 one, multiplied 
 * with the correct XPNf functions to arrive at eps^-1 alpha^0. 
 */

#include <4IntPNF.h>
#include <Auxiliaries.h>
#include <cmath>
#include <XFactorPNf.h>
#include <Input_Common.h>

/*
 * the leading (eps^-2) terms vanish, since if x is the additional
 * integration the leading eps-term of this plus-dist vanishes,
 * and if x is delta'ed, it contributes a 1/alpha
 */

//fourth integration is over x

double PNfI4EM1x(double w, double s6, double x, double c) {
	return (mexp*log(FA1(v(w),tl(s6),y(x),0,0,b(c))*FA2(v(w),tl(s6),y(x),0,0,b(c))*FB1(v(w),tl(s6),y(x),0,0,b(c))*FB2(v(w),tl(s6),y(x),0,0,b(c))*pow(FA1(v(w),tl(s6),0,0,0,b(c)),-1)*pow(FA2(v(w),tl(s6),0,0,0,b(c)),-1)*
       pow(FB1(v(w),tl(s6),0,0,0,b(c)),-1)*pow(FB2(v(w),tl(s6),0,0,0,b(c)),-1))*pow(x,-1)*XPNf0(w,s6,0,0,c))/2.;
}

double PNfI4E0x(double w, double s6, double x, double c) {
	return (mexp*pow(x,-1)*(-2*(pow(log(FA1(v(w),tl(s6),0,0,0,b(c))),2) - pow(log(FA1(v(w),tl(s6),y(x),0,0,b(c))),2) + pow(log(FA2(v(w),tl(s6),0,0,0,b(c))),2) - pow(log(FA2(v(w),tl(s6),y(x),0,0,b(c))),2) + 
          pow(log(FB1(v(w),tl(s6),0,0,0,b(c))),2) - pow(log(FB1(v(w),tl(s6),y(x),0,0,b(c))),2) + pow(log(FB2(v(w),tl(s6),0,0,0,b(c))),2) - pow(log(FB2(v(w),tl(s6),y(x),0,0,b(c))),2))*XPNf0(w,s6,0,0,c) + 
       log(FA1(v(w),tl(s6),y(x),0,0,b(c))*FA2(v(w),tl(s6),y(x),0,0,b(c))*FB1(v(w),tl(s6),y(x),0,0,b(c))*FB2(v(w),tl(s6),y(x),0,0,b(c))*pow(FA1(v(w),tl(s6),0,0,0,b(c)),-1)*pow(FA2(v(w),tl(s6),0,0,0,b(c)),-1)*
          pow(FB1(v(w),tl(s6),0,0,0,b(c)),-1)*pow(FB2(v(w),tl(s6),0,0,0,b(c)),-1))*XPNf1(w,s6,0,0,c)))/2.;
}

//fourth integration is over s5

double PNfI4EM1s(double w, double s6, double s5, double c) {
	return (pow(s5,-1)*(log(FA1(v(w),tl(s6),0,0,0,b(c))*FA2(v(w),tl(s6),0,0,0,b(c))*FB1(v(w),tl(s6),0,0,0,b(c))*FB2(v(w),tl(s6),0,0,0,b(c)))*XPNf0(w,s6,0,0,c) - 
       log(FA1(v(w),tl(s6),0,t5(s5),0,b(c))*FA2(v(w),tl(s6),0,t5(s5),0,b(c))*FB1(v(w),tl(s6),0,t5(s5),0,b(c))*FB2(v(w),tl(s6),0,t5(s5),0,b(c)))*XPNf0(w,s6,s5,0,c)))/2.;
}

double PNfI4E0s(double w, double s6, double s5, double c) {
	return (pow(s5,-1)*(-2*log(s5)*log(FA1(v(w),tl(s6),0,0,0,b(c))*FA2(v(w),tl(s6),0,0,0,b(c))*FB1(v(w),tl(s6),0,0,0,b(c))*FB2(v(w),tl(s6),0,0,0,b(c)))*XPNf0(w,s6,0,0,c) + 
       4*(pow(log(FA1(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FA2(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FB1(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FB2(v(w),tl(s6),0,0,0,b(c))),2))*XPNf0(w,s6,0,0,c) + 
       2*log(s5)*log(FA1(v(w),tl(s6),0,t5(s5),0,b(c))*FA2(v(w),tl(s6),0,t5(s5),0,b(c))*FB1(v(w),tl(s6),0,t5(s5),0,b(c))*FB2(v(w),tl(s6),0,t5(s5),0,b(c)))*XPNf0(w,s6,s5,0,c) - 
       4*(pow(log(FA1(v(w),tl(s6),0,t5(s5),0,b(c))),2) + pow(log(FA2(v(w),tl(s6),0,t5(s5),0,b(c))),2) + pow(log(FB1(v(w),tl(s6),0,t5(s5),0,b(c))),2) + pow(log(FB2(v(w),tl(s6),0,t5(s5),0,b(c))),2))*
        XPNf0(w,s6,s5,0,c) + log(FA1(v(w),tl(s6),0,0,0,b(c))*FA2(v(w),tl(s6),0,0,0,b(c))*FB1(v(w),tl(s6),0,0,0,b(c))*FB2(v(w),tl(s6),0,0,0,b(c)))*XPNf1(w,s6,0,0,c) - 
       log(FA1(v(w),tl(s6),0,t5(s5),0,b(c))*FA2(v(w),tl(s6),0,t5(s5),0,b(c))*FB1(v(w),tl(s6),0,t5(s5),0,b(c))*FB2(v(w),tl(s6),0,t5(s5),0,b(c)))*XPNf1(w,s6,s5,0,c)))/2.;
}

//fourth integration is over z

double PNfI4EM1z(double w, double s6, double z, double c) {
	return pow(z,-1)*(log(FA1(v(w),tl(s6),0,0,0,b(c))*FA2(v(w),tl(s6),0,0,0,b(c))*FB1(v(w),tl(s6),0,0,0,b(c))*FB2(v(w),tl(s6),0,0,0,b(c)))*XPNf0(w,s6,0,0,c) - 
     log(FA1(v(w),tl(s6),0,0,u(z),b(c))*FA2(v(w),tl(s6),0,0,u(z),b(c))*FB1(v(w),tl(s6),0,0,u(z),b(c))*FB2(v(w),tl(s6),0,0,u(z),b(c)))*XPNf0(w,s6,0,z,c));
}

double PNfI4E0z(double w, double s6, double z, double c) {
	return pow(z,-1)*(-4*log(z)*log(FA1(v(w),tl(s6),0,0,0,b(c))*FA2(v(w),tl(s6),0,0,0,b(c))*FB1(v(w),tl(s6),0,0,0,b(c))*FB2(v(w),tl(s6),0,0,0,b(c)))*XPNf0(w,s6,0,0,c) + 
     4*(pow(log(FA1(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FA2(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FB1(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FB2(v(w),tl(s6),0,0,0,b(c))),2))*XPNf0(w,s6,0,0,c) + 
     4*log(z)*log(FA1(v(w),tl(s6),0,0,u(z),b(c))*FA2(v(w),tl(s6),0,0,u(z),b(c))*FB1(v(w),tl(s6),0,0,u(z),b(c))*FB2(v(w),tl(s6),0,0,u(z),b(c)))*XPNf0(w,s6,0,z,c) - 
     4*(pow(log(FA1(v(w),tl(s6),0,0,u(z),b(c))),2) + pow(log(FA2(v(w),tl(s6),0,0,u(z),b(c))),2) + pow(log(FB1(v(w),tl(s6),0,0,u(z),b(c))),2) + pow(log(FB2(v(w),tl(s6),0,0,u(z),b(c))),2))*XPNf0(w,s6,0,z,c) + 
     log(FA1(v(w),tl(s6),0,0,0,b(c))*FA2(v(w),tl(s6),0,0,0,b(c))*FB1(v(w),tl(s6),0,0,0,b(c))*FB2(v(w),tl(s6),0,0,0,b(c)))*XPNf1(w,s6,0,0,c) - 
     log(FA1(v(w),tl(s6),0,0,u(z),b(c))*FA2(v(w),tl(s6),0,0,u(z),b(c))*FB1(v(w),tl(s6),0,0,u(z),b(c))*FB2(v(w),tl(s6),0,0,u(z),b(c)))*XPNf1(w,s6,0,z,c));
}
